/// <reference types="node" />
import { TSessionStore, TXmlParser, TErrorHandler, TWall, TShield, TGuard, TTaskScheduler } from "../types";
import { ViewEngine, ResultMapper, ComponentOption } from "../abstracts";
import * as http from "http";
import { TaskSchedulerManager } from "../utils";
import { Logger } from "./logger";
import { IDTOValidator, IEtagOption, IFolderMap, IControllerRoute, ICacheStore, IScheduleTaskInput } from "../interfaces";
export declare class App {
    private logger_;
    private viewEngine_;
    /**
     * sessionStore class, default - MemorySessionStore
     *
     * @type {TSessionStore}
     * @memberof App
     */
    sessionStore: TSessionStore;
    /**
     * XmlParser class - used to parse the xml
     *
     * @type {TXmlParser}
     * @memberof App
     */
    xmlParser: TXmlParser;
    /**
     * Whether to parse cookie or not, default - true
     * If false, then session wont work.
     *
     * @type {boolean}
     * @memberof App
     */
    shouldParseCookie: boolean;
    /**
     * Whether to parse the http body data in post request, default - true
     *
     * @type {boolean}
     * @memberof App
     */
    shouldParseBody: boolean;
    /**
     * session timeout in minute - default is 60 minute
     *
     * @type {number}
     * @memberof App
     */
    sessionTimeOut: number;
    /**
     *  name of application - default is fort. Visible in header and cookie.
     * Change name if you dont want any one to know the framework name.
     *
     * @type {string}
     * @memberof App
     */
    appName: string;
    /**
     * Views folder location. By default it is - views.
     *
     * @type {string}
     * @memberof App
     */
    viewPath: string;
    private componentOption_;
    private cacheManager_;
    walls: TWall[];
    /**
     * keep alive timeout in millisecond for requests, default is 72000
     *
     * @memberof App
     */
    keepAliveTimeout: number;
    /**
     * eTag Settings
     *
     * @type {IEtagOption}
     * @memberof App
     */
    eTag: IEtagOption;
    private folders_;
    set logger(value: Logger);
    get logger(): Logger;
    shields: TShield[];
    guards: TGuard[];
    /**
     * port at which app will listen, default - 4000
     *
     * @private
     * @memberof App
     */
    port: number;
    errorHandler: TErrorHandler;
    private appSessionIdentifier_;
    set routes(value: IControllerRoute[]);
    /**
     * view engine use to render the view
     *
     * @
     * @memberof Fort
     */
    set viewEngine(value: typeof ViewEngine);
    set resultMapper(value: typeof ResultMapper);
    set componentOption(value: typeof ComponentOption);
    /**
     * folders which should be visible to requests. By default nothing is allowed.
     *
     * @
     * @memberof Fort
     */
    set folders(value: IFolderMap[]);
    httpServer: http.Server;
    create(): Promise<void>;
    onNewRequest: (request: any, response: any) => void;
    destroy(): Promise<void>;
    validator: IDTOValidator;
    crons: IScheduleTaskInput[];
    scheduler: TaskSchedulerManager;
    useCache: boolean;
    cacheStore: ICacheStore;
    cronJobScheduler: TTaskScheduler;
}
export declare const Fort: App;
